/*
Copyright 2016 FirstMate LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
'use strict';

import React, {
    Children,
    Component,
    PropTypes,
} from 'react';
import {
    DeviceEventEmitter,
    Event,
    requireNativeComponent,
    View
} from 'react-native';

const positionType = PropTypes.shape({
    lat: PropTypes.number.isRequired,
    lon: PropTypes.number.isRequired,
})

var OSMDroidMapView = requireNativeComponent('OSMDroidMapView', {
    name: 'OSMDroidMapView',
    propTypes: {
        ...View.propTypes,
        tileSource: PropTypes.shape({
            name: PropTypes.string,
            zoomMin: PropTypes.number,
            zoomMax: PropTypes.number,
            tileSizePixels: PropTypes.number,
            imageTileNameEnding: PropTypes.string,
            baseURL: PropTypes.string,
            urlFormat: PropTypes.string
        }),
        onMapChange: PropTypes.shape({
            onScroll: PropTypes.func,
            onZoom: PropTypes.func,
        }),
        center: positionType,
    }
});

class MapView extends Component {
    constructor(props) {
        super(props);
        this._handleOnScroll = this._handleOnScroll.bind(this);
        this._handleOnZoom = this._handleOnZoom.bind(this);
    }

    componentWillMount() {
        DeviceEventEmitter.addListener('onScroll', this._handleOnScroll);
        DeviceEventEmitter.addListener('onZoom', this._handleOnZoom);
    }

    _handleOnScroll(event) {
        if (this.props.onMapChange && this.props.onMapChange.onScroll) {
            this.props.onMapChange.onScroll(event);
        }
    }

    _handleOnZoom(event) {
        if (this.props.onMapChange && this.props.onMapChange.onZoom) {
            this.props.onMapChange.onZoom(event);
        }
    }

    _compareTileSources(first, second) {
        if (!first || !second) return false;

        return (first.name === second.name)
            && (first.zoomMin === second.zoomMin)
            && (first.zoomMax === second.zoomMax)
            && (first.tileSizePixels === second.tileSizePixels)
            && (first.imageTileNameEnding === second.imageTileNameEnding)
            && (first.baseURL === second.baseURL)
            && (first.urlFormat === second.urlFormat);
    }

    _comparePositions(first, second) {
        if (!first || !second) return false;

        return (first.lat === second.lat) && (first.lon === second.lon);
    }

    render() {
        return (<OSMDroidMapView {...this.props}/>);
    }
}

export default MapView;
