package com.reactnativeosmdroid;

import org.osmdroid.tileprovider.tilesource.TileSourceFactory;
import org.osmdroid.views.MapView;
import org.osmdroid.tileprovider.tilesource.OnlineTileSourceBase;
import org.osmdroid.tileprovider.MapTile;
import org.osmdroid.api.IMapController;
import org.osmdroid.util.GeoPoint;
import org.osmdroid.events.MapListener;
import org.osmdroid.events.ScrollEvent;
import org.osmdroid.events.ZoomEvent;
import org.osmdroid.events.DelayedMapListener;
import org.osmdroid.api.IGeoPoint;

import com.facebook.react.uimanager.SimpleViewManager;
import com.facebook.react.uimanager.ThemedReactContext;
import com.facebook.react.uimanager.annotations.ReactProp;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.uimanager.events.RCTEventEmitter;
import com.facebook.react.common.MapBuilder;
import com.facebook.react.modules.core.DeviceEventManagerModule;

import java.util.Map;
import javax.annotation.Nullable;
import android.support.v4.content.ContextCompat;
import android.util.Log;

/*
Copyright 2016 FirstMate LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/


public class ReactOSMDroidManager extends SimpleViewManager<MapView> {
    public static final String REACT_CLASS = "OSMDroidMapView";
    final GeoPoint defaultPoint = new GeoPoint(41.308294, -72.927931);

    @Override
    public String getName() {
        return REACT_CLASS;
    }

    @Override
    public MapView createViewInstance(ThemedReactContext context) {
        final MapView map = new MapView(context);

        map.setTileSource(TileSourceFactory.MAPNIK);

        IMapController mapController = map.getController();
        mapController.setZoom(16);
        mapController.setCenter(defaultPoint);

        map.setMultiTouchControls(true);

        map.setMapListener(new DelayedMapListener(new MapListener(){
                public boolean onScroll(ScrollEvent event) {
                    WritableMap eventData = Arguments.createMap();
                    eventData.putInt("x", event.getX());
                    eventData.putInt("y", event.getY());

                    IGeoPoint center = map.getMapCenter();
                    eventData.putDouble("lat", center.getLatitude());
                    eventData.putDouble("lon", center.getLongitude());

                    ReactContext reactContext = (ReactContext)map.getContext();
                    reactContext
                        .getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
                        .emit("onScroll", eventData);

                    return true;
                }

                public boolean onZoom(ZoomEvent event) {
                    WritableMap eventData = Arguments.createMap();
                    eventData.putInt("zoomLevel", event.getZoomLevel());

                    IGeoPoint center = map.getMapCenter();
                    eventData.putDouble("lat", center.getLatitude());
                    eventData.putDouble("lon", center.getLongitude());

                    ReactContext reactContext = (ReactContext)map.getContext();
                    reactContext
                        .getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
                        .emit("onZoom", eventData);

                    return true;
                }
            }, 100));


        Log.i("ReactNativeOSMDroid", "Created new MapView");

        return map;
    }

    @ReactProp(name="tileSource")
    public void setTileSource(MapView view, ReadableMap parameters) {
        Log.i("ReactNativeOSMDroid", "setTileSource");
        final String name = parameters.getString("name");
        final int zoomMin = parameters.hasKey("zoomMin") ? parameters.getInt("zoomMin") : 0;
        final int zoomMax = parameters.hasKey("zoomMax") ? parameters.getInt("zoomMax") : 18;
        final int tileSizePixels = parameters.hasKey("tileSizePixels") ? parameters.getInt("tileSizePixels") : 256;
        final String imageTileNameEnding = parameters.hasKey("imageTileNameEnding") ? parameters.getString("imageTileNameEnding") : "";
        final String[] baseURL = new String[] {parameters.getString("baseURL")};
        final String urlFormat = parameters.hasKey("urlFormat") ? parameters.getString("urlFormat") : "zxy";

        view.setTileSource(new OnlineTileSourceBase(name, zoomMin,
            zoomMax, tileSizePixels, imageTileNameEnding, baseURL) {
                @Override
                public String getTileURLString(MapTile aTile) {
                    String urlSuffix = "";

                    if (urlFormat.equals("zxy")) {
                        urlSuffix = aTile.getZoomLevel() + "/" + aTile.getX() + "/" + aTile.getY();
                    } else if (urlFormat.equals("zyx")) {
                        urlSuffix = aTile.getZoomLevel() + "/" + aTile.getY() + "/" + aTile.getX();
                    }

                    return getBaseUrl() + urlSuffix + mImageFilenameEnding;
                }
            });
    }

    @ReactProp(name="center")
    public void setCenter(MapView map, @Nullable ReadableMap point) {
        // point will be null if the prop has been removed, so
        // just leave the map where it is.
        if (point == null) {
            return;
        }

        GeoPoint newCenter = new GeoPoint(point.getDouble("lat"), point.getDouble("lon"));
        GeoPoint oldCenter = (GeoPoint)map.getMapCenter();
        Log.i("ReactNativeOSMDroid", "Old: " + oldCenter.toString() + "\nNew: " + newCenter.toString());
        if (oldCenter.getLatitude() != newCenter.getLatitude() || oldCenter.getLongitude() != newCenter.getLongitude()) {
            map.getController().setCenter(newCenter);
            Log.i("ReactNativeOSMDroid", "Set new center");
        }
    }
}
